.macpack atari

.segment "RUN0"
  scrcode "     SEA DRAGON     "
  .byte 24+64
  .byte $FF
  .byte <run
  .byte >run

.segment "ROM0"

  zp_source     = $20
  zp_dest       = $22
  zp_length     = $24

.proc run
  sei
  lda #$00
  sta NMIEN
  sta DMACLT
  sta IRQEN

  ldx #$FF
  txs

  jsr reset_HW
  jsr clear_RAM
  jsr copy_to_RAM

  lda #<SYSVBV      ; configure own immediate VBLANK vector
  sta VVBLKI
  lda #>SYSVBV
  sta VVBLKI+1

  lda #<XITVBV      ; configure own deferred VBLANK vector
  sta VVBLKD
  lda #>XITVBV
  sta VVBLKD+1

  lda #<MY_VBI      ; set real immediate VBLANK vector
  sta $202
  lda #>MY_VBI
  sta $203

  lda #$02			; release POKEY from reset
  sta SKCTL

  lda #$FE
  sta DRKMSK
  lda #$00
  sta COLRSH

  lda #$04			; select keypad and start POTs
  sta CONSOL
  sta POTGO

  jmp $480
.endproc

.proc reset_HW
  lda #$00          ; reset h/w
  tax
hw_reset_loop:
  sta HPOSP0,x
  sta AUDF1,x
  sta DMACLT,x
  inx
  bne hw_reset_loop
  rts
.endproc

.proc clear_RAM
  lda #$00          ; clear RAM $200..$3FFF
  sta zp_dest
  lda #$02
  sta zp_dest+1

  lda #$00
  ldy #$00
loop:
  sta (zp_dest),y
  iny
  bne loop
  inc zp_dest+1
  ldx zp_dest+1
  cpx #$40
  bne loop
  rts
.endproc

.proc copy_to_RAM
  lda #<RAM_IMAGE
  sta zp_source
  lda #>RAM_IMAGE
  sta zp_source+1

  lda #<RAM_START
  sta zp_dest
  lda #>RAM_START
  sta zp_dest+1

  lda #<(RAM_END-RAM_START)
  sta zp_length
  lda #>(RAM_END-RAM_START)
  sta zp_length+1

  ldx #$00

copy:
  lda (zp_source,x)
  sta (zp_dest,x)
  inc zp_source
  bne skip1
  inc zp_source+1

skip1:
  inc zp_dest
  bne skip2
  inc zp_dest+1

skip2:
  dec zp_length
  lda zp_length
  cmp #$FF
  bne skip3
  dec zp_length+1

skip3:
  lda zp_length
  bne copy
  lda zp_length+1
  bne copy

  rts
.endproc

; vertical blank interrupt routine
.proc MY_VBI
  pha
  txa
  pha
  tya
  pha

  lda consol_delay	; hold CONSOL value for a number of frames
  bmi skip
  dec consol_delay
  bpl skip
  lda #$07
  sta MY_CONSOL
skip:

  lda SKSTAT        ; read keyboard if key held
  and #$04
  bne no_key_press

  lda KBCODE
  lsr
  and #$0F

  cmp #$04
  bne no_reset
  jmp ($FFFC)

no_reset:
  tax
  lda CONSOL_MAPPINGS,x
  bne consol_key_press
  stx CH
  jmp no_key_press

consol_key_press:
  sta MY_CONSOL
  lda #$07
  sta consol_delay

no_key_press:
  ldx #$03
next_trigger:
  lda TRIG0,x			; read joystick trigger
  sta STRIG0,x
  dex
  bpl next_trigger

  ldx #$00
  ldy #$00
next_stick:
  lda POT0,y
  cmp #114+50
  rol STICK0,x
  cmp #114-50
  rol STICK0,x
  lda POT1,y
  cmp #114+50
  rol STICK0,x
  cmp #114-50
  rol STICK0,x
  lda STICK0,x
  eor #2+8
  and #$0F
  sta STICK0,x
  iny
  iny
  inx
  cpx #$04
  bne next_stick

  lda #$04			; select keypad and start POTs
  sta CONSOL
  sta POTGO

  jmp (VVBLKI)

CONSOL_MAPPINGS:
      ;   # 0 * R 9 8 7 P 6 5 4 S 3 2 1
      ; 0 1 2 3 4 5 6 7 8 9 a b c d e f
  .byte 0,5,0,3,0,0,0,0,0,0,0,0,6,0,0,0

.endproc

; system vertical blank processing
.proc SYSVBV
  lda CHBAS
  sta CHBASE

  lda SDLSTL
  sta DLISTL
  lda SDLSTH
  sta DLISTH

  lda SDMCTL
  sta DMACLT

  lda GPRIOR
  sta PRIOR

  ldx #$08
colour_loop:
  lda PCOLR0,x
  sta COLPM0,x
  dex
  bpl colour_loop

  inc RTCLOK+2
  bne clocked
  inc RTCLOK+1
  bne clocked
  inc RTCLOK

clocked:

          LDX     #0         ; POINT TO TIMER1
          JSR     DCTIMR     ; GO DECREMENT TIMER1
          BNE     SYSVB2     ; BRANCH IF STILL COUNTING
          JSR     JTIMR1     ; GO JUMP TO ROUTINE
SYSVB2:   LDX     #2         ; POINT TO TIMER 2
          JSR     DCTIMR
          BNE     SYSVB4     ; IF DIDNT GO ZERO
          JSR     JTIMR2     ; GO JUMP TO TIMER2 ROUTINE
SYSVB4:   LDX     #2         ; RESTORE X
SYSVBB:   INX
          INX
          LDA     CDTMV1,X
          ORA     CDTMV1+1,X
          BEQ     SYSVBA
          JSR     DCTIMR     ; DECREMENT AND SET FLAG IF NONZERO
          STA     CDTMF3-4,X
SYSVBA:   CPX     #8         ; SEE IF DONE ALL 3
          BNE     SYSVBB     ; LOOP

  jmp (VVBLKD)

JTIMR1:   JMP	  (CDTMA1)
JTIMR2:   JMP	  (CDTMA2)
DCTIMR:   LDY	  CDTMV1,X	 ; LO BYTE
		  BNE	  DCTIM1	 ; NONZERO, GO DEC IT
		  LDY	  CDTMV1+1,X ; SEE IF BOTH ZERO
		  BEQ	  DCTXF		 ; YES, EXIT NONZERO
		  DEC	  CDTMV1+1,X ; DEC HI BYTE
DCTIM1:   DEC	  CDTMV1,X	 ; DEC LO BYTE
		  BNE	  DCTXF
		  LDY	  CDTMV1+1,X
		  BNE	  DCTXF
		  LDA	  #0		 ; WENT ZERO. RETURN ZERO
		  RTS
DCTXF:    LDA	  #$FF		  ; RETURN NONZERO
		  RTS
.endproc

; exit vertical blank
.proc XITVBV
  pla
  tay
  pla
  tax
  pla
  rti
.endproc

; 
; D:sea.s
; 
         .include "sea.001"
         .include "sea.002"
         .include "sea.003"
         .include "sea.004"
         .include "sea.005"
         .include "sea.006"
         .include "sea.007"
         .include "sea.008"
         .include "sea.009"
;
         
